{% load i18n %}
<div class="session_select actions">
    {% for field in session_select_form %}
        {% if field.label %}<label>{{ field.label }} {% endif %}{{ field }}{% if field.label %}</label>{% endif %}
    {% endfor %}
    <button type="submit" class="button" name="run_sessions" title="{% trans "Run the selected session" %}">{% trans "Run" %}</button>
    {% if session_count.all %}
        <button type="submit" class="button" name="clear_sessions" title="{% trans "Clear session-infos" %}">{% trans "Clear session-infos" %}</button>
    {% endif %}
    {% if session_count.all %}{% include "minke/session_summary.html" %}{% endif %}
    <a class='message-toggle' data-show='{% trans "show messages" %}' data-hide='{% trans "hide messages" %}'>{% trans "hide messages" %}</a>
</div>
