{% extends 'admin/change_list.html' %}
{% load static admin_list minke %}

{% block extrahead %}
    {{ block.super }}
    <script type="text/javascript" src="{% static 'minke/js/minke.js' %}"></script>
{% endblock %}

{% block date_hierarchy %}
    {{ block.super }}
    {% if display_session_switch %}{% minke_session_switch %}{% endif %}
{% endblock %}

{% block result_list %}
    {% if action_form and actions_on_top and cl.show_admin_actions %}
        {% admin_actions %}
        {% if display_session_select %}{% minke_session_select %}{% endif %}
    {% endif %}
    {% if display_session_info %}{% minke_result_list cl %}{% else %}{% result_list cl %}{% endif %}
    {% if action_form and actions_on_bottom and cl.show_admin_actions %}
        {% admin_actions %}
        {% if display_session_select %}{% minke_session_select %}{% endif %}
    {% endif %}
{% endblock %}
