# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-19 17:03
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('minke', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='host',
            options={'ordering': ['host']},
        ),
    ]
